@@ -4,9 +4,11 @@ setwd(tempdir())
 
 library(SFSI)
 data(wheatHTP)
-X = scale(X[1:400,])/sqrt(ncol(X))   # Subset and scale markers
-G = tcrossprod(X)                    # Genomic relationship matrix
-y = as.vector(scale(Y[1:400,"YLD"])) # Subset response variable
+
+index = which(Y$CV %in% 1:2)
+M = scale(M[index,])/sqrt(ncol(M))   # Subset and scale markers
+G = tcrossprod(M)                    # Genomic relationship matrix
+y = as.vector(scale(Y[index,"E1"])) # Subset response variable
 
 # Calculate heritability using all data
 fm1 = fitBLUP(y,K=G)
@@ -25,7 +27,7 @@ cor(y,fm1$u)        # G-BLUP accuracy
 cor(y,fitted(fm3))  # SSI accuracy
 
 # Predicting a testing set using training set
-tst = sample(seq_along(y),ceiling(0.3*length(y)))
+tst = seq(1,length(y),by=3)
 trn = (seq_along(y))[-tst]
 
 # Calculate heritability in training data
